@@ -21,6 +21,7 @@ import gobject
 import time
 
 import os 
+import tempfile
 
 import dbus
 import dbus.service
@@ -31,7 +32,7 @@ from synceKPM.dataserver.rapiutil import *
 
 import synceKPM.constants
 
-
+    
 class DataServer(dbus.service.Object):
     def __init__(self, busConn, dataServerEventLoop):
         dbus.service.Object.__init__(self, busConn, synceKPM.constants.DBUS_SYNCEKPM_DATASERVER_OBJPATH)
@@ -74,6 +75,7 @@ class DataServer(dbus.service.Object):
         self._storageInformation = None
 
         self.deviceName   = ""
+        self.deviceHalObjPath = ""
 
 
 
@@ -171,10 +173,22 @@ class DataServer(dbus.service.Object):
         deviceObject = dbus.SystemBus().get_object("org.freedesktop.Hal",obj_path)
         device = dbus.Interface(deviceObject,"org.freedesktop.Hal.Device")
 
+        if device.PropertyExists("pda.pocketpc.connection_blocked"):
+            if device.GetPropertyInteger("pda.pocketpc.connection_blocked") == 1:
+                device = None
+                print "\n\nPlease disable your firewall. Problem with outgoing connection\n\n"
+                return 
+            if device.GetPropertyInteger("pda.pocketpc.connection_blocked") == 2:
+                device = None
+                print "\n\nPlease disable your firewall. Problem with incoming connection\n\n"
+                return 
+
         if not device.PropertyExists("pda.pocketpc.name"):
             device = None
             return
         self.hal_device = device
+        self.deviceHalObjPath = obj_path
+
         self.hal_device_synce = dbus.Interface(deviceObject,"org.freedesktop.Hal.Device.Synce")
 
         self.deviceName = self.hal_device.GetPropertyString("pda.pocketpc.name")
@@ -214,13 +228,15 @@ class DataServer(dbus.service.Object):
 
 
     def hal_device_disconnected_cb(self, obj_path):
-        print "disconnecting device ", self.deviceName 
-        self.deviceDisconnected( self.deviceName )
-        self.hal_device = None
-        self.hal_device_synce = None
-        self.deviceName   = ""
-        self.deviceIsConnected = False
-        self._programList = []
+        if obj_path == self.deviceHalObjPath:
+            print "disconnecting device ", self.deviceName , " obj_path = ", obj_path
+            self.deviceHalObjPath = ""
+            self.deviceDisconnected( self.deviceName )
+            self.hal_device = None
+            self.hal_device_synce = None
+            self.deviceName   = ""
+            self.deviceIsConnected = False
+            self._programList = []
         pass
 
 
@@ -239,6 +255,15 @@ class DataServer(dbus.service.Object):
         
         self.updateDeviceInformation()
 
+
+        try:
+            self.busConn.get_object("org.synce.SyncEngine", "/org/synce/SyncEngine")
+            self.handleSyncEngineStatusChange( True, True ) 
+        except dbus.DBusException:
+            self.handleSyncEngineStatusChange( False, False) 
+
+
+
         gobject.timeout_add(45000, self.updateDeviceInformation)
         
         pass
@@ -248,14 +273,52 @@ class DataServer(dbus.service.Object):
             return False
 
 
+        self.updateSyncPicture()
+
         self.updatePowerStatus()
         self.updateListInstalledPrograms()
         self.updateStorageInformation()
         #self.updatePartnerships()
+
         
         return True
 
 
+    def updateSyncPicture(self):
+        if not self.deviceIsConnected:
+            return False
+
+        rapi_session = RAPISession(None, 0)
+        
+        #If the file does not exist, then we are finished
+        try:
+            SyncIcoOnDevice = rapi_session.file_open("/Windows/sync.ico") 
+
+            SyncIcoOnDeviceContents = SyncIcoOnDevice.read() 
+            SyncIcoOnDevice.close() 
+
+            fd,filename = tempfile.mkstemp( '.ico' )
+            os.close(fd)
+
+            f = open( filename, 'w' )
+
+            f.write(SyncIcoOnDeviceContents)
+            f.close() 
+            self.DeviceDisplayPicture( filename) 
+
+        except RAPIError, e:
+            print "Failed to get sync.ico file"
+            return
+
+
+
+       
+    @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='s')
+    def DeviceDisplayPicture(self, fileContents):
+        pass
+
+
+
 
 
     @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='a(ssttt)')
@@ -269,7 +332,7 @@ class DataServer(dbus.service.Object):
 
         if self.deviceIsConnected:
             #Amount free on mainMemory:
-            rapi_session = RAPISession(0)
+            rapi_session = RAPISession(None, 0)
             freeDisk,totalDisk,freeDiskTotal = rapi_session.getDiskFreeSpaceEx("\\")
             self._storageInformation.append( ("MainMemory","\\" ,freeDisk,totalDisk,freeDiskTotal) )
 
@@ -280,7 +343,7 @@ class DataServer(dbus.service.Object):
                     try:
                         freeDisk,totalDisk,freeDiskTotal = rapi_session.getDiskFreeSpaceEx("\\"+folder["Name"]+"\\")
                         self._storageInformation.append( ("/%s"%folder["Name"],"\\%s\\"%folder["Name"], freeDisk,totalDisk,freeDiskTotal) )
-                    except pyrapi2.RapiError, e:
+                    except RAPIError, e:
                         print "Failed to get disk space for %s, skipping: %d: %s" % (folder["Name"], e.err_code, e)
 
 
@@ -454,7 +517,7 @@ class DataServer(dbus.service.Object):
             dbus.SystemBus().get_object("org.synce.odccm", "/org/synce/odccm/DeviceManager")
             self.handleOdccmStatusChange( True ) 
         except dbus.DBusException:
-            print "odccm is NOT running!!"
+            pass
 
 
         # connect to Hal device manager
@@ -479,7 +542,6 @@ class DataServer(dbus.service.Object):
             self.handleSyncEngineStatusChange( True, True ) 
         except dbus.DBusException:
             self.handleSyncEngineStatusChange( False, False) 
-            print "SyncEngine is NOT running!!"
 
      
     @dbus.service.method('org.synce.kpm.DataServerInterface')
@@ -491,7 +553,7 @@ class DataServer(dbus.service.Object):
 
         if self.deviceIsConnected:
             try:
-                rapi_session = RAPISession(0)
+                rapi_session = RAPISession(None, 0)
                 _programs = config_query_get(rapi_session, None ,   "UnInstall").children.values()
                 for program in _programs:
                     self._programList.append( program.type )
@@ -519,7 +581,7 @@ class DataServer(dbus.service.Object):
 
         
         try:
-            rapi_session = RAPISession(0)
+            rapi_session = RAPISession(None, 0)
             self._powerStatus = rapi_session.getSystemPowerStatus(True)
         except Exception,e:
             print "Error while retrieving battery information:"
@@ -548,7 +610,7 @@ class DataServer(dbus.service.Object):
 
         if self.deviceIsConnected:
             try:
-                rapi_session = RAPISession(0)
+                rapi_session = RAPISession(None, 0)
                 hkcu = rapi_session.HKEY_CURRENT_USER
                 owner_key = hkcu.open_sub_key(r"ControlPanel\Owner")
                 owner = owner_key.query_value("Name")
@@ -584,7 +646,7 @@ class DataServer(dbus.service.Object):
         node.setProp("value", "1") ; 
 
         print "_config_query: CeProcessConfig request is \n", doc_node.serialize("utf-8",1)
-        rapi_session = RAPISession(0)
+        rapi_session = RAPISession(None, 0)
         reply = rapi_session.process_config(doc_node.serialize("utf-8",0), 1)
         reply_doc = libxml2.parseDoc(reply)
         print "_config_query: CeProcessConfig response is \n", reply_doc.serialize("utf-8",1)
@@ -605,41 +667,44 @@ class DataServer(dbus.service.Object):
         
         RAPI_BUFFER_SIZE = 65535
         
-        rapi_session = RAPISession(0)
-        
-        fileHandle = rapi_session.createFile("%s%s"%(copyToDirectory,fileName), GENERIC_WRITE , 0, CREATE_ALWAYS , FILE_ATTRIBUTE_NORMAL )
-        
-        if fileHandle != 0xffffffff:
-            print "Copying file to device"
+        rapi_session = RAPISession(None, 0)
 
-            fileObject = open(localFilenameAndPath,"rb")
-            read_buffer = fileObject.read(RAPI_BUFFER_SIZE)
+        try:
+            fileHandle = rapi_session.file_open("%s%s"%(copyToDirectory,fileName), "w", 0, FILE_ATTRIBUTE_NORMAL )
+        except RAPIError, e:
+            print "Failed to open destination file: %s" % e
+            return
 
-            bytesWritten = 0 
+        print "Copying file to device"
 
+        fileObject = open(localFilenameAndPath,"rb")
+        read_buffer = fileObject.read(RAPI_BUFFER_SIZE)
+
+        bytesWritten = 0 
+
+        self.installationCopyProgress( (100*bytesWritten) / fileSize )
+
+        while len(read_buffer) > 0:
+            fileHandle.write(read_buffer)
+            bytesWritten += len(read_buffer) 
             self.installationCopyProgress( (100*bytesWritten) / fileSize )
 
-            while len(read_buffer) > 0:
-                rapi_session.writeFile( fileHandle, read_buffer, len(read_buffer) )
-                bytesWritten += len(read_buffer) 
-                self.installationCopyProgress( (100*bytesWritten) / fileSize )
-
-                read_buffer = fileObject.read(RAPI_BUFFER_SIZE)
+            read_buffer = fileObject.read(RAPI_BUFFER_SIZE)
 
 
-            print "Closing filehandle..."
-            returnValue = rapi_session.closeHandle(fileHandle)
+        print "Closing filehandle..."
+        fileHandle.close()
 
-            fileObject.close()
+        fileObject.close()
+
+        applicationName = "wceload.exe"
+        #Add the quotes to make sure paths with spaces are no problem
+        applicationParms = "\"%s%s\""%(copyToDirectory,fileName)
+        if not deleteCab:
+            applicationParms += " /nodelete"
+        result = rapi_session.createProcess( applicationName, applicationParms )
+        self.CabInstallStarted()
 
-            applicationName = "wceload.exe"
-            #Add the quotes to make sure paths with spaces are no problem
-            applicationParms = "\"%s%s\""%(copyToDirectory,fileName)
-            if not deleteCab:
-                applicationParms += " /nodelete"
-            result = rapi_session.createProcess( applicationName, applicationParms )
-            self.CabInstallStarted()
-         
     
     @dbus.service.method('org.synce.kpm.DataServerInterface', in_signature="sau")
     def createPartnership(self, name, items):
@@ -777,4 +842,155 @@ class DataServer(dbus.service.Object):
 
 
 
+    
+
+    @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='i')
+    def registry_keys_finished(self, key_id):
+        pass
+
+    @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='ia(si)')
+    def registry_keys(self, key_id, sub_keys_and_child_count):
+        pass
+
+    @dbus.service.method('org.synce.kpm.DataServerInterface', in_signature="is")
+    def query_registry_keys(self, key_id, key_path):
+        
+        rapi_session = RAPISession(None, 0)
+ 
+        root_key = None 
+        sub_key_path = ""
+
+        if key_path.startswith("/HKEY_CLASSES_ROOT"):
+            root_key = rapi_session.HKEY_CLASSES_ROOT
+            sub_key_path = key_path[len("/HKEY_CLASSES_ROOT"):]
+        elif key_path.startswith("/HKEY_CURRENT_USER"):
+            root_key = rapi_session.HKEY_CURRENT_USER
+            sub_key_path = key_path[len("/HKEY_CURRENT_USER"):]
+        elif key_path.startswith("/HKEY_LOCAL_MACHINE"):
+            root_key = rapi_session.HKEY_LOCAL_MACHINE
+            sub_key_path = key_path[len("/HKEY_LOCAL_MACHINE"):]
+
+        if root_key is None:
+            print key_path, " is not recognized as valid starting path"
+       
+        
+        sub_key_path = sub_key_path.replace("/", "\\")
+        
+        my_key = root_key.open_sub_key(sub_key_path)
+        
+        subkeys_and_child_count = my_key.keys_and_childcount(True)
+        my_key.close()
+
+
+        #self.registry_keys( key_id , my_subkeys )
+        self.registry_keys( key_id , subkeys_and_child_count)
+        self.registry_keys_finished( key_id )
+
+        pass
+
+    
+    @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='i')
+    def registry_finished_fetching_values(self, key_id):
+        pass
+
+    @dbus.service.signal('org.synce.kpm.DataServerInterface', signature='ia(siii)axasaiaiai')
+    def registry_values(self, key_id, list_value_name_type_numerical_dataindex, list_numerical_data, list_string_data, list_binary_data, list_binary_data_from, list_binary_data_to):
+        pass
+    
+    
+    @dbus.service.method('org.synce.kpm.DataServerInterface', in_signature="is")
+    def query_registry_values(self, key_id, key_path):
+        #print "[DataServer] Querying all values for key with path: ", key_path
+        
+        rapi_session = RAPISession(None, 0)
+ 
+        root_key = None 
+        sub_key_path = ""
+
+        if key_path.startswith("/HKEY_CLASSES_ROOT"):
+            root_key = rapi_session.HKEY_CLASSES_ROOT
+            sub_key_path = key_path[len("/HKEY_CLASSES_ROOT"):]
+        elif key_path.startswith("/HKEY_CURRENT_USER"):
+            root_key = rapi_session.HKEY_CURRENT_USER
+            sub_key_path = key_path[len("/HKEY_CURRENT_USER"):]
+        elif key_path.startswith("/HKEY_LOCAL_MACHINE"):
+            root_key = rapi_session.HKEY_LOCAL_MACHINE
+            sub_key_path = key_path[len("/HKEY_LOCAL_MACHINE"):]
+
+        if root_key is None:
+            print "[ERROR] : ", path_of_key, " is not recognized as valid starting path"
+       
+        
+        sub_key_path = sub_key_path.replace("/", "\\")
+        
+        my_key = root_key.open_sub_key(sub_key_path)
+        
+        values = [] 
+        list_numerical_data = []
+        list_string_data = []
+        list_binary_data = []
+
+        list_binary_data_from = []
+        list_binary_data_to   = []
+
+        index_in_container_binary_data = 0
+
+        index_numerical_data = 0 
+        index_string_data = 0 
+        index_binary_data = 0 ; 
+        
+        for (value_name,value_type,value_data) in my_key.values():
+            #print "value name = ", value_name, "   value_type=", value_type
+            value_data_type = 0
+
+            if value_type == REG_DWORD:
+                value_data_type = 1
+            if value_type == REG_DWORD_BIG_ENDIAN:
+                value_data_type = 1
+            if value_type == REG_BINARY:
+                value_data_type = 2
+            if value_type == REG_MULTI_SZ:
+                value_data_type = 2
+           
+            
+            index_data = 0 
+
+            if value_data_type == 0:
+                index_data = index_string_data
+                index_string_data += 1
+                
+                list_string_data.append( value_data )
+            
+            if value_data_type == 1:
+                index_data = index_numerical_data
+                index_numerical_data += 1
+
+                list_numerical_data.append( value_data )
+            
+            if value_data_type == 2:
+                index_data = index_binary_data
+                index_binary_data += 1
+
+                list_binary_data_from.append( index_in_container_binary_data )
+               
+                for i in value_data:
+                    list_binary_data.append(ord(i))
+                    index_in_container_binary_data += 1 ; 
+
+                list_binary_data_to.append(index_in_container_binary_data)
+
+
+
+
+            values.append( ( value_name, value_type, value_data_type, index_data ) )
+
+
+
+        my_key.close()
+
+        self.registry_values ( key_id , values, list_numerical_data, list_string_data, list_binary_data, list_binary_data_from, list_binary_data_to ) 
+
+        pass
+
+
 
